/*******************************************************************************
 **3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
 **      10        20        30        40        50        60        70        80
 **
 ** notify-osd
 **
 ** util.c - all sorts of helper functions
 **
 ** Copyright 2009 Canonical Ltd.
 **
 ** Authors:
 **    Cody Russell <cody.russell@canonical.com>
 **
 ** This program is free software: you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License version 3, as published
 ** by the Free Software Foundation.
 **
 ** This program is distributed in the hope that it will be useful, but
 ** WITHOUT ANY WARRANTY; without even the implied warranties of
 ** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 ** PURPOSE.  See the GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License along
 ** with this program.  If not, see <http://www.gnu.org/licenses/>.
 **
 *******************************************************************************/

#include <glib.h>
#include <pango/pango.h>

#define CHARACTER_LT_REGEX              "&(lt;|#60;|#x3C;)"
#define CHARACTER_GT_REGEX              "&(gt;|#62;|#x3e;)"
#define CHARACTER_AMP_REGEX             "&(amp;|#38;|#x26;)"
#define CHARACTER_APOS_REGEX            "&(apos;|#39;|#x27;)"
#define CHARACTER_QUOT_REGEX            "&(quot;|#34;|#x22;)"

#define LINK_MATCH_REGEX                "<a[^>]*?href\\s*=\\s*[\"\"']?([^'\"\" >]+?)['\"\"]?>(.*?)</a>"
#define LINK_REPLACE_REGEX              "<a[^>]*?href\\s*=\\s*[\"\"']?([^'\"\" >]+?)['\"\"]?>|</a>"
#define OTHER_MATCH_REGEX               "<([A-Z][A-Z0-9]*)\\b[^>]*>(.*?)</\\1>"
#define OTHER_REPLACE_REGEX             "<([A-Z][A-Z0-9]*)\\b[^>]*>|</[A-Z]>"

static gchar*
strip_html (const gchar *text, const gchar *match_regex, const gchar* replace_regex)
{
	GRegex   *regex;
	gchar    *ret;
	gboolean  match = FALSE;
	GMatchInfo *info = NULL;

	regex = g_regex_new (match_regex, G_REGEX_CASELESS | G_REGEX_OPTIMIZE, 0, NULL);
	match = g_regex_match (regex, text, 0, &info);
	g_regex_unref (regex);

	if (match) {
		regex = g_regex_new (replace_regex, G_REGEX_CASELESS | G_REGEX_OPTIMIZE, 0, NULL);
		ret = g_regex_replace (regex, text, -1, 0, "", 0, NULL);
		g_regex_unref (regex);
	} else {
		ret = g_strdup (text);
	}

	return ret;
}

static gchar*
replace_symbol (const gchar *text, const gchar *match_regex, const gchar *replace_text)
{
	GRegex *regex;
	gchar  *ret;

	regex = g_regex_new (match_regex, G_REGEX_CASELESS | G_REGEX_OPTIMIZE, 0, NULL);
	ret = g_regex_replace (regex, text, -1, 0, replace_text, 0, NULL);
	g_regex_unref (regex);

	return ret;
}

gchar*
filter_text (const gchar *text)
{
	gchar *text1;
	gchar *text2;
	gchar *text3;
	gchar *text4;
	gchar *text5;
	gchar *text6;
	gchar *text7;

	text1 = strip_html (text, LINK_MATCH_REGEX, LINK_REPLACE_REGEX);

	text2 = strip_html (text1, OTHER_MATCH_REGEX, OTHER_REPLACE_REGEX);
	g_free (text1);

	text3 = replace_symbol (text2, CHARACTER_AMP_REGEX, "&");
	g_free (text2);

	text4 = replace_symbol (text3, CHARACTER_LT_REGEX, "<");
	g_free (text3);

	text5 = replace_symbol (text4, CHARACTER_GT_REGEX, ">");
	g_free (text4);

	text6 = replace_symbol (text5, CHARACTER_APOS_REGEX, "'");
	g_free (text5);

	text7 = replace_symbol (text6, CHARACTER_QUOT_REGEX, "\"");
	g_free (text6);

	return text7;
}

